var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { commonsTypeHasPropertyBoolean } from 'tscommons-es-core';
import { CommonsAiIntegerOutputNeuralNetwork } from './commons-ai-integer-output-neural-network';
export class CommonsAiBooleanOutputNeuralNetwork extends CommonsAiIntegerOutputNeuralNetwork {
    constructor(humaniseOutput) {
        super(1, 0, humaniseOutput);
    }
    trainForBooleans(examples, hiddenLayers, iterations, errorThresh) {
        return __awaiter(this, void 0, void 0, function* () {
            const converted = new Map();
            converted.set(1, examples.get(true) || []);
            converted.set(0, examples.get(false) || []);
            return yield this.trainForIntegers(converted, hiddenLayers, iterations, errorThresh);
        });
    }
    runForBoolean(query) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.runWithConfidence(query);
            return {
                output: result.output === 1 ? true : false,
                confidence: result.confidence
            };
        });
    }
    testForBooleans(examples) {
        return __awaiter(this, void 0, void 0, function* () {
            const results = {
                valid: new Map(),
                invalid: new Map()
            };
            for (const value of examples.keys()) {
                for (const example of examples.get(value)) {
                    const result = yield this.runForBoolean(example);
                    if (result.output === value)
                        results.valid.set(example, result);
                    else
                        results.invalid.set(example, result);
                }
            }
            return results;
        });
    }
    isCorrectOutput(o, additionalData) {
        if (!commonsTypeHasPropertyBoolean(additionalData, 'goal'))
            throw new Error('No goal in additional data. This should not be possible');
        const typecast = additionalData;
        const goal = typecast.goal ? 1 : 0;
        return o === goal;
    }
    fittestEvolution(ai, ao, bi, bo, additionalData) {
        if (!commonsTypeHasPropertyBoolean(additionalData, 'goal'))
            throw new Error('No goal in additional data. This should not be possible');
        const typecast = additionalData;
        const goal = typecast.goal ? 1 : 0;
        if (ao.output === goal && bo.output !== goal)
            return ai;
        if (ao.output !== goal && bo.output === goal)
            return bi;
        // both are the same output
        if (ao.output === goal) {
            // smallest confidence wins (i.e. best match)
            if (ao.confidence < bo.confidence)
                return ai;
            return bi;
        }
        else {
            // largest confidence wins (i.e. worst match)
            if (ao.confidence > bo.confidence)
                return ai;
            return bi;
        }
    }
    dreamForBoolean(goal, maxIterations, errorThreshold, additionalData) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.dream(maxIterations, errorThreshold, Object.assign(Object.assign({}, additionalData), { goal: goal }));
        });
    }
}
//# sourceMappingURL=commons-ai-boolean-output-neural-network.js.map