var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { commonsOutputDoing, commonsOutputProgress, commonsOutputResult, commonsOutputDebug } from 'nodecommons-es-cli';
import { CommonsAiNeuralNetwork } from './commons-ai-neural-network';
export class CommonsAiConfidenceNeuralNetwork extends CommonsAiNeuralNetwork {
    processOut(_data) {
        // this method won't be used, as we use processOutWithConfidence instead
        throw new Error('Method not applicable. This should not be called.');
    }
    runWithConfidence(data) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.net)
                throw new Error('No net loaded');
            const mapped = yield this.convertInputData(data);
            if (this.validate)
                CommonsAiNeuralNetwork.validateData0To1(mapped);
            const typecast = mapped;
            const output = this.net.run(typecast);
            return yield this.processOutWithConfidence(output);
        });
    }
    generateNoise(_additionalData) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Dreaming is not implemented for this class');
        });
    }
    mutate(_src, _additionalData) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Dreaming is not implemented for this class');
        });
    }
    isCorrectOutput(_o, _additionalData) {
        throw new Error('Dreaming is not implemented for this class');
    }
    fittestEvolution(_ai, _ao, _bi, _bo, _additionalData) {
        throw new Error('Dreaming is not implemented for this class');
    }
    dream(maxIterations, errorThreshold, additionalData) {
        return __awaiter(this, void 0, void 0, function* () {
            let evolution = yield this.generateNoise(additionalData);
            let evolutionScore = yield this.runWithConfidence(evolution);
            if (errorThreshold !== undefined
                && this.isCorrectOutput(evolutionScore.output, additionalData)
                && evolutionScore.confidence < errorThreshold) {
                commonsOutputDebug('No need to dream, already within error threshold');
                return evolution;
            }
            commonsOutputDoing('Dreaming');
            for (let ttl = maxIterations; ttl-- > 0;) {
                if (ttl % 100 === 0)
                    commonsOutputProgress(`${ttl} remaining iterations, ${evolutionScore.confidence} confidence`);
                const mutation = yield this.mutate(evolution, additionalData);
                const mutationScore = yield this.runWithConfidence(mutation);
                const fittest = this.fittestEvolution(evolution, evolutionScore, mutation, mutationScore, additionalData);
                if (fittest === mutation) {
                    evolution = mutation;
                    evolutionScore = mutationScore;
                    if (errorThreshold !== undefined
                        && this.isCorrectOutput(evolutionScore.output, additionalData)
                        && evolutionScore.confidence < errorThreshold) {
                        commonsOutputResult(`threshold met with confidence ${evolutionScore.confidence}`);
                        return mutation;
                    }
                }
            }
            commonsOutputResult(`best confidence ${evolutionScore.confidence}`);
            return evolution;
        });
    }
}
//# sourceMappingURL=commons-ai-confidence-neural-network.js.map