var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { CommonsAiIntegerOutputNeuralNetwork } from './commons-ai-integer-output-neural-network';
export class CommonsAiEnumOutputNeuralNetwork extends CommonsAiIntegerOutputNeuralNetwork {
    constructor(enums, humaniseOutput) {
        super(enums.length - 1, 0, humaniseOutput);
        this.enums = enums;
        if (enums.length < 2)
            throw new Error('CommonsAiEnumOutputNeuralNetwork requires an enum with at least 2 items');
    }
    trainForEnums(examples, hiddenLayers, iterations, errorThresh) {
        return __awaiter(this, void 0, void 0, function* () {
            const converted = new Map();
            for (const e of this.enums) {
                converted.set(this.enums.indexOf(e), examples.get(e) || []);
            }
            return yield this.trainForIntegers(converted, hiddenLayers, iterations, errorThresh);
        });
    }
    runForEnum(query) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.runWithConfidence(query);
            if (result.output < 0 || result.output >= this.enums.length)
                throw new Error('Invalid enum index returned');
            return {
                output: this.enums[result.output],
                confidence: result.confidence
            };
        });
    }
    testForEnums(examples) {
        return __awaiter(this, void 0, void 0, function* () {
            const results = {
                valid: new Map(),
                invalid: new Map()
            };
            for (const value of examples.keys()) {
                for (const example of examples.get(value)) {
                    const result = yield this.runForEnum(example);
                    if (result.output === value)
                        results.valid.set(example, result);
                    else
                        results.invalid.set(example, result);
                }
            }
            return results;
        });
    }
}
//# sourceMappingURL=commons-ai-enum-output-neural-network.js.map