var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { commonsAiDataHumanise } from '../helpers/commons-ai-data';
import { CommonsAiNeuralNetwork } from './commons-ai-neural-network';
export class CommonsAiFloatOutputNeuralNetwork extends CommonsAiNeuralNetwork {
    constructor(max, min = 0, humaniseOutput) {
        super();
        this.max = max;
        this.min = min;
        this.humaniseOutput = humaniseOutput;
        this.delta = this.max - this.min;
        if (this.delta <= 0)
            throw new Error('Invalid min/max range for CommonsAiFloatOutputNeuralNetwork');
        if (this.humaniseOutput && this.humaniseOutput < 0)
            throw new Error('Humanised output is negative');
    }
    processIn(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const processed = [];
            for (const d of data) {
                const numbers = yield this.convertInputData(d.input);
                if (this.validate)
                    CommonsAiNeuralNetwork.validateData0To1(numbers);
                let value = (d.output - this.min) / this.delta;
                if (this.humaniseOutput) {
                    const safeHumaniseOutput = this.humaniseOutput / this.delta;
                    value = commonsAiDataHumanise(value, safeHumaniseOutput);
                }
                processed.push({
                    input: numbers,
                    output: [value]
                });
            }
            return processed;
        });
    }
    processOut(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const out = data['0'];
            return (out * this.delta) + this.min;
        });
    }
}
//# sourceMappingURL=commons-ai-float-output-neural-network.js.map