var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { commonsAiDataHumanise } from '../helpers/commons-ai-data';
import { CommonsAiConfidenceNeuralNetwork } from './commons-ai-confidence-neural-network';
import { CommonsAiNeuralNetwork } from './commons-ai-neural-network';
export class CommonsAiIntegerOutputNeuralNetwork extends CommonsAiConfidenceNeuralNetwork {
    constructor(max, min = 0, humaniseOutput) {
        super();
        this.max = max;
        this.min = min;
        this.humaniseOutput = humaniseOutput;
        if (this.max !== Math.round(this.max))
            throw new Error('CommonsAiIntegerOutputNeuralNetwork max is not an integer');
        if (this.min !== Math.round(this.min))
            throw new Error('CommonsAiIntegerOutputNeuralNetwork min is not an integer');
        this.delta = this.max - this.min;
        if (this.delta <= 0)
            throw new Error('Invalid min/max range for CommonsAiIntegerOutputNeuralNetwork');
        if (this.humaniseOutput && this.humaniseOutput < 0)
            throw new Error('Humanised output is negative');
    }
    processIn(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const processed = [];
            for (const d of data) {
                const numbers = yield this.convertInputData(d.input);
                if (this.validate)
                    CommonsAiNeuralNetwork.validateData0To1(numbers);
                let value = (d.output - this.min) / this.delta;
                if (this.humaniseOutput) {
                    const safeHumaniseOutput = this.humaniseOutput / this.delta;
                    value = commonsAiDataHumanise(value, safeHumaniseOutput);
                }
                processed.push({
                    input: numbers,
                    output: [value]
                });
            }
            return processed;
        });
    }
    processOutWithConfidence(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const out = data['0'];
            const multiplied = out * this.delta;
            const rounded = Math.round(multiplied);
            return {
                output: rounded + this.min,
                confidence: Math.abs(multiplied - rounded)
            };
        });
    }
    trainForIntegers(examples, hiddenLayers, iterations, errorThresh, crossValidate = true) {
        return __awaiter(this, void 0, void 0, function* () {
            const trainingData = [];
            for (const value of examples.keys()) {
                if (value < this.min || value > this.max)
                    throw new Error(`Invalid value for CommonsAiIntegerOutputNeuralNetwork`);
                trainingData.push(...examples.get(value)
                    .map((example) => ({
                    input: example,
                    output: value
                })));
            }
            return yield this.train(trainingData, hiddenLayers, iterations, errorThresh, crossValidate);
        });
    }
    runForInteger(query) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.runWithConfidence(query);
        });
    }
    testForIntegers(examples) {
        return __awaiter(this, void 0, void 0, function* () {
            const results = {
                valid: new Map(),
                invalid: new Map()
            };
            for (const value of examples.keys()) {
                for (const example of examples.get(value)) {
                    const result = yield this.runForInteger(example);
                    if (result.output === value)
                        results.valid.set(example, result);
                    else
                        results.invalid.set(example, result);
                }
            }
            return results;
        });
    }
}
//# sourceMappingURL=commons-ai-integer-output-neural-network.js.map