import brain from 'brain.js/src/index';
import { TCommonsAiTrainingData } from '../types/tcommons-ai-training-data';
import { TCommonsAiGenericTrainingData } from '../types/tcommons-ai-generic-training-data';
import { CommonsAiTrainingLogger } from './commons-ai-training-logger';
export declare type TBrainTrainingData = {
    input: brain.NeuralNetworkInput;
    output: brain.NeuralNetworkOutput;
};
export declare abstract class CommonsAiNeuralNetwork<I, O> {
    protected static validateData0To1(data: number[] | {
        [key: string]: number;
    }): void | never;
    protected net: brain.NeuralNetwork | undefined;
    protected validate: boolean;
    private loggerConstructor;
    protected abstract convertInputData(data: I): Promise<number[] | {
        [key: string]: number;
    }>;
    protected abstract processIn(data: TCommonsAiGenericTrainingData<I, O>[]): Promise<TCommonsAiTrainingData[]>;
    protected abstract processOut(data: {}): Promise<O>;
    setValidate(validate: boolean): void;
    setLoggerConstructor(callback: (instance: CommonsAiNeuralNetwork<any, any>) => CommonsAiTrainingLogger): void;
    train(data: TCommonsAiGenericTrainingData<I, O>[], hiddenLayers: number[], iterations?: number, errorThresh?: number, crossValidate?: boolean): Promise<void>;
    run(data: I): Promise<O>;
    load(filename: string): Promise<void>;
    save(filename: string): Promise<void>;
}
