var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { commonsAiDataHumanise } from '../helpers/commons-ai-data';
import { commonsAiObjectBest } from '../helpers/commons-ai-object';
import { CommonsAiNeuralNetwork } from './commons-ai-neural-network';
export class CommonsAiObjectOutputNeuralNetwork extends CommonsAiNeuralNetwork {
    constructor(enumsValues, humaniseOutput) {
        super();
        this.enumsValues = enumsValues;
        this.humaniseOutput = humaniseOutput;
    }
    processIn(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const processed = [];
            for (const d of data) {
                const numbers = yield this.convertInputData(d.input);
                if (this.validate)
                    CommonsAiNeuralNetwork.validateData0To1(numbers);
                const clone = Object.assign({}, d.output);
                if (this.humaniseOutput) {
                    for (const e of this.enumsValues) {
                        if (clone[e] === undefined)
                            continue;
                        clone[e] = commonsAiDataHumanise(clone[e], this.humaniseOutput);
                    }
                }
                processed.push({
                    input: numbers,
                    output: clone
                });
            }
            return processed;
        });
    }
    processOut(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const object = Object.assign({}, data);
            for (const e of this.enumsValues) {
                if (object[e] === undefined)
                    object[e] = 0;
            }
            return object;
        });
    }
    trainForSingularEnumObjects(examples, hiddenLayers, iterations, errorThresh, crossValidate = true) {
        return __awaiter(this, void 0, void 0, function* () {
            const trainingData = [];
            for (const e of examples.keys()) {
                if (!this.enumsValues.includes(e))
                    throw new Error('Invalid enum value for CommonsAiObjectOutputNeuralNetwork');
                for (const v of examples.get(e)) {
                    const object = {};
                    for (const e2 of this.enumsValues)
                        object[e2] = e === e2 ? 1 : 0;
                    trainingData.push({
                        input: v,
                        output: object
                    });
                }
            }
            return yield this.train(trainingData, hiddenLayers, iterations, errorThresh, crossValidate);
        });
    }
    testForSingularEnumObjects(examples) {
        return __awaiter(this, void 0, void 0, function* () {
            const results = {
                valid: new Map(),
                invalid: new Map()
            };
            for (const value of examples.keys()) {
                for (const example of examples.get(value)) {
                    const result = yield this.run(example);
                    const best = commonsAiObjectBest(result);
                    if (best === value)
                        results.valid.set(example, result);
                    else
                        results.invalid.set(example, result);
                }
            }
            return results;
        });
    }
}
//# sourceMappingURL=commons-ai-object-output-neural-network.js.map