import { commonsOutputDoing, commonsOutputProgress, commonsOutputSuccess, commonsOutputResult } from 'nodecommons-es-cli';
export class CommonsAiTrainingLogger {
    constructor(instance, interval = 500) {
        this.instance = instance;
        this.interval = interval;
        this.kfolds = 1;
        this.iterations = 0;
        this.completed = 0;
        this.lastDisplayTime = 0;
    }
    displayStart() {
        commonsOutputDoing('Cross-training neural network');
    }
    displayProgress(kfold, kfoldIterations, totalIterations, kfoldError, bestError) {
        commonsOutputProgress(`${kfold} kfolds; ${kfoldIterations} of ${totalIterations} iterations; error: ${kfoldError.toFixed(6)} vs ${bestError.toFixed(6)}`);
    }
    displayBest(best) {
        if (!best)
            commonsOutputSuccess();
        else
            commonsOutputResult(`best error ${best.toFixed(8)}`);
    }
    start() {
        this.displayStart();
        this.lastDisplayTime = new Date().getTime();
    }
    update(log) {
        if (log.iterations < this.iterations) {
            this.error = undefined;
            this.completed += this.iterations;
            this.kfolds++;
        }
        this.iterations = log.iterations;
        this.error = log.error;
        this.best = this.best === undefined ? this.error : Math.min(this.best, this.error);
        const now = new Date().getTime();
        const delta = now - this.lastDisplayTime;
        if (delta >= this.interval) {
            this.displayProgress(this.kfolds, this.iterations, this.completed + this.iterations, this.error, this.best);
            this.lastDisplayTime = now;
        }
    }
    done() {
        if (!this.best)
            this.displayBest();
        else
            this.displayBest(this.best);
    }
}
//# sourceMappingURL=commons-ai-training-logger.js.map